"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startDevServer = startDevServer;

var _plugins = require("./plugins");

var _util = require("./util");

var _Server = _interopRequireDefault(require("webpack-dev-server/lib/Server"));

var _createLogger = _interopRequireDefault(require("webpack-dev-server/lib/utils/createLogger"));

var _PromiseDefer = _interopRequireDefault(require("./PromiseDefer"));

/**************************************************
 * Created by nanyuantingfeng on 23/08/2017 14:29.
 **************************************************/
function colorInfo(msg) {
  return "\x1B[1m\x1B[34m".concat(msg, "\x1B[39m\x1B[22m");
}

function colorError(msg) {
  return "\x1B[1m\x1B[31m".concat(msg, "\x1B[39m\x1B[22m");
}

process.on('unhandledRejection', function (err) {
  throw err;
});

function startDevServer(context) {
  var webpackConfig = context.webpackConfig;
  var firstWpOpt = Array.isArray(webpackConfig) ? webpackConfig[0] : webpackConfig;
  var options = webpackConfig.devServer || firstWpOpt.devServer || {};
  var log = (0, _createLogger.default)(options);

  _Server.default.addDevServerEntrypoints(webpackConfig, options);

  var defer = (0, _PromiseDefer.default)();
  var compiler;

  try {
    compiler = (0, _plugins.webpack)(webpackConfig);
  } catch (e) {
    if (e instanceof _plugins.WebpackOptionsValidationError) {
      log.error(colorError(e.message));
      process.exit(1);
    }

    defer.reject(e);
  }

  var server;

  try {
    server = new _Server.default(compiler, options, log);
  } catch (e) {
    var OptionsValidationError = require('webpack-dev-server/lib/OptionsValidationError');

    if (e instanceof OptionsValidationError) {
      log.error(colorError(e.message));
      process.exit(1);
    }

    defer.reject(e);
  }

  ;
  ['SIGINT', 'SIGTERM'].forEach(function (sig) {
    process.on(sig, function () {
      server.close();
      process.exit();
    });
  });
  server.listen(options.port, options.host, function (err) {
    if (err) throw err;
    defer.resolve(server);
    log.info("\nService is running at ".concat(colorInfo((0, _util.createDomain)(options))));
  });
  return defer.promise;
}