"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _os = _interopRequireDefault(require("os"));

var _path = _interopRequireDefault(require("path"));

var _plugins = require("./plugins");

/**************************************************
 * Created by nanyuantingfeng on 27/10/2017 16:28.
 **************************************************/
var EXCLUDE_REG_NODE_MODULES = /[/\\\\]node_modules[/\\\\]/;

function fnGetThemeMap(packageMap, cwd) {
  var theme = {};
  var packageMapTheme = packageMap.theme;

  if (packageMapTheme && typeof packageMapTheme === 'string') {
    var pp = packageMapTheme;

    if (pp.charAt(0) === '.') {
      pp = _path.default.resolve(cwd, pp);
    }

    var fn = require(pp);

    theme = fn();
  } else if (packageMapTheme && (0, _typeof2.default)(packageMapTheme) === 'object') {
    theme = packageMapTheme;
  }

  return theme;
}

function XSX_LOADER(context) {
  var _context$enableHappyp = context.enableHappypack,
      enableHappypack = _context$enableHappyp === void 0 ? true : _context$enableHappyp;
  return enableHappypack ? happypackLoaders(context) : commonLoaders(context, false);
}

function happypackLoaders(context) {
  var _commonLoaders = commonLoaders(context, true),
      JSX_LOADER = _commonLoaders.JSX_LOADER,
      TSX_LOADER = _commonLoaders.TSX_LOADER;

  var THREAD_POOL_CPU_SIZE = _os.default.cpus().length;

  var threadPool = _plugins.HappyPack.ThreadPool({
    size: THREAD_POOL_CPU_SIZE
  });

  var plugins = context.plugins;
  plugins.push(new _plugins.HappyPack({
    id: 'jsx',
    threadPool: threadPool,
    loaders: JSX_LOADER
  }));
  plugins.push(new _plugins.HappyPack({
    id: 'tsx',
    threadPool: threadPool,
    loaders: TSX_LOADER
  }));
  return {
    JSX_LOADER: [{
      loader: 'happypack/loader',
      options: {
        id: 'jsx'
      }
    }],
    TSX_LOADER: [{
      loader: 'happypack/loader',
      options: {
        id: 'tsx'
      }
    }]
  };
}

function commonLoaders(context, enableHappypack) {
  var babelOptions = context.babelOptions,
      tsOptions = context.tsOptions;
  var JSX_LOADER = [{
    loader: 'babel-loader',
    options: babelOptions
    /*, getReplaceLodashLoader()*/

  }];
  var tsOptions2 = tsOptions;

  if (enableHappypack) {
    tsOptions2 = (0, _objectSpread2.default)({
      happyPackMode: true
    }, tsOptions);
  }

  var TSX_LOADER = [{
    loader: 'ts-loader',
    options: tsOptions2
  }];
  return {
    JSX_LOADER: JSX_LOADER,
    TSX_LOADER: TSX_LOADER
  };
}

function _default(_x, _x2) {
  return _ref.apply(this, arguments);
}

function _ref() {
  _ref = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee(context, next) {
    var cwd, _context$limit, limit, ENV, packageMap, hash, plugins, theme, postcssOptions, rules, workerFileName, _XSX_LOADER, JSX_LOADER, TSX_LOADER, scriptRules, stylesRules, othersRules, filename, chunkFilename;

    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            context.rules = [];
            next();
            cwd = context.cwd, _context$limit = context.limit, limit = _context$limit === void 0 ? 10240 : _context$limit, ENV = context.ENV, packageMap = context.packageMap, hash = context.hash, plugins = context.plugins;
            theme = fnGetThemeMap(packageMap, cwd);
            postcssOptions = context.postcssOptions, rules = context.rules;
            workerFileName = hash ? '[name]-[hash].worker.js' : '[name].worker.js';
            _XSX_LOADER = XSX_LOADER(context), JSX_LOADER = _XSX_LOADER.JSX_LOADER, TSX_LOADER = _XSX_LOADER.TSX_LOADER;
            scriptRules = [{
              test: /\.worker\.jsx?$/,
              exclude: [EXCLUDE_REG_NODE_MODULES],
              use: [{
                loader: 'worker-loader',
                options: {
                  name: workerFileName
                }
              }].concat((0, _toConsumableArray2.default)(JSX_LOADER))
            }, {
              test: function test(filePath) {
                return /\.lazy\.jsx?$/.test(filePath);
              },
              exclude: [EXCLUDE_REG_NODE_MODULES],
              use: [{
                loader: 'bundle-loader',
                options: {
                  lazy: true
                }
              }].concat((0, _toConsumableArray2.default)(JSX_LOADER))
            }, {
              test: function test(filePath) {
                return /\.jsx?$/.test(filePath) && !/\.lazy\.jsx?$/.test(filePath) && !/\.worker\.jsx?$/.test(filePath);
              },
              exclude: [EXCLUDE_REG_NODE_MODULES],
              use: JSX_LOADER
            }, {
              test: /\.tsx?$/,
              exclude: [EXCLUDE_REG_NODE_MODULES],
              use: TSX_LOADER
            }];
            stylesRules = [{
              test: function test(filePath) {
                return /\.css$/.test(filePath) && !/\.module\.css$/.test(filePath);
              },
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }]
            }, {
              test: /\.module\.css$/,
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]'
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }]
            }, {
              test: function test(filePath) {
                return /\.less$/.test(filePath) && !/\.module\.less$/.test(filePath);
              },
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }, {
                loader: 'less-loader',
                options: {
                  javascriptEnabled: true,
                  sourceMap: true,
                  modifyVars: theme
                }
              }]
            }, {
              test: /\.module\.less$/,
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]'
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }, {
                loader: 'less-loader',
                options: {
                  javascriptEnabled: true,
                  sourceMap: true,
                  modifyVars: theme
                }
              }]
            }];
            othersRules = [{
              test: /\.woff(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/font-woff'
                }
              }]
            }, {
              test: /\.woff2(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/font-woff'
                }
              }]
            }, {
              test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/octet-stream'
                }
              }]
            }, {
              test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/vnd.ms-fontobject'
                }
              }]
            }, {
              test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'image/svg+xml'
                }
              }]
            }, {
              test: /\.svgx$/,
              use: [{
                loader: '@svgr/webpack'
              }]
            }, {
              test: /\.json5$/,
              use: [{
                loader: 'json5-loader'
              }]
            }, {
              test: /\.(png|jpg|jpeg|gif)(\?v=\d+\.\d+\.\d+)?$/i,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit
                }
              }]
            }, {
              test: /\.html?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  name: '[path]-[name].[ext]'
                }
              }]
            }, {
              test: /\.hbs?$/,
              use: [{
                loader: 'mustache-loader'
              }]
            }];

            if (ENV.isProduction || ENV.isBeta) {
              stylesRules.map(function (rule) {
                return rule.use[0] = _plugins.MiniCSSExtractPlugin.loader;
              });
              filename = hash ? '[name]-[hash].css' : '[name].css';
              chunkFilename = hash ? '[id]-[hash].css' : '[id].css';
              plugins.push(new _plugins.MiniCSSExtractPlugin({
                filename: filename,
                chunkFilename: chunkFilename
              }));
            }

            context.rules = scriptRules.concat(stylesRules).concat(othersRules).concat(rules);
            plugins.push(new _plugins.WatchIgnorePlugin([/\.d\.ts$/]));

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _ref.apply(this, arguments);
}